@@ -1,7 +1,7 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2024 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2025 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
  * Copyright 2004 Dag Lem <resid@nimrod.no>
  *
@@ -24,6 +24,7 @@
 #define FILTER_H
 
 #include "FilterModelConfig.h"
+#include "Voice.h"
 
 #include "siddefs-fp.h"
 
@@ -41,7 +42,6 @@ private:
     unsigned short** resonance;
     unsigned short** volume;
 
-protected:
     FilterModelConfig& fmc;
 
     /// Current filter/voice mixer setting.
@@ -56,6 +56,7 @@ protected:
     /// Current volume amplifier setting.
     unsigned short* currentVolume = nullptr;
 
+protected:
     /// Filter highpass state.
     int Vhp = 0;
 
@@ -65,6 +66,7 @@ protected:
     /// Filter lowpass state.
     int Vlp = 0;
 
+private:
     /// Filter external input.
     int Ve = 0;
 
@@ -82,6 +84,7 @@ protected:
     /// Switch voice 3 off.
     bool voice3off = false;
 
+protected:
     /// Highpass, bandpass, and lowpass filter modes.
     //@{
     bool hp = false;
@@ -99,6 +102,12 @@ private:
     /// Selects which inputs to route through filter.
     unsigned char filt = 0;
 
+private:
+    inline int getNormalizedVoice(Voice& v) const
+    {
+        return fmc.getNormalizedVoice(v.output(), v.envelope()->output());
+    }
+
 protected:
     /**
      * Update filter cutoff frequency.
@@ -120,7 +129,9 @@ protected:
     /**
      * Get the filter cutoff register value
      */
-    unsigned int getFC() const { return fc; }
+    inline unsigned int getFC() const { return fc; }
+
+    virtual int solveIntegrators() = 0;
 
 public:
     Filter(FilterModelConfig& fmc);
@@ -133,9 +144,9 @@ public:
      * @param v1 voice 1 in
      * @param v2 voice 2 in
      * @param v3 voice 3 in
-     * @return filtered output
+     * @return filtered output, unsigned 16 bit
      */
-    virtual unsigned short clock(int v1, int v2, int v3) = 0;
+    unsigned short clock(Voice& v1, Voice& v2, Voice& v3);
 
     /**
      * Enable filter.
@@ -183,10 +194,40 @@ public:
      * @param input a signed 16 bit sample
      */
     void input(short input) { Ve = fmc.getNormalizedVoice(input/32768.f, 0); }
-
-    inline int getNormalizedVoice(float value, unsigned int env) const { return fmc.getNormalizedVoice(value, env); }
 };
 
 } // namespace reSIDfp
 
+#if RESID_INLINING || defined(FILTER_CPP)
+
+namespace reSIDfp
+{
+
+RESID_INLINE
+unsigned short Filter::clock(Voice& voice1, Voice& voice2, Voice& voice3)
+{
+    const int V1 = getNormalizedVoice(voice1);
+    const int V2 = getNormalizedVoice(voice2);
+    // Voice 3 is silenced by voice3off if it is not routed through the filter.
+    const int V3 = (filt3 || !voice3off) ? getNormalizedVoice(voice3) : 0;
+
+    int Vsum = 0;
+    int Vmix = 0;
+
+    (filt1 ? Vsum : Vmix) += V1;
+    (filt2 ? Vsum : Vmix) += V2;
+    (filt3 ? Vsum : Vmix) += V3;
+    (filtE ? Vsum : Vmix) += Ve;
+
+    Vhp = currentSummer[currentResonance[Vbp] + Vlp + Vsum];
+
+    Vmix += solveIntegrators();
+
+    return currentVolume[currentMixer[Vmix]];
+}
+
+#endif
+
+} // namespace reSIDfp
+
 #endif
